#include <iostream>
#include <cmath>
#include <vector>

using matrix = std::vector<std::vector<double>>;
using rvec   = std::vector<double>;

rvec solveGauss(matrix& A, rvec& B) {

  int N(B.size());
  int M(A.size());
  rvec res(N,0.);

  if (M == 0 or A[0].size() != N) {
    std::cerr << "Incompatible shapes of matrix and vector" << std::endl
              << "Solution will not be computed" << std::endl;
  }
  else {

    int r = -1;
    for (int j=0; j<N; j++) {

      // Get argmax of column j
      int    k    = 0;
      double vmax = fabs(A[k][j]);
      for (int i = r+1; i<M; i++)
        if (fabs(A[i][j])>vmax) {
          vmax = fabs(A[i][j]);
          k    = i;
        }
        if (fabs(vmax)>1.e-20) {
          // Next pivot
          r++;
          // Normalize line
          for (int jj=0; jj<N; jj++) {
            A[k][jj] /= vmax;
          }
          B[k] /= vmax;

          // Swap r and k lines
          double tmp;
          for (int jj=0; jj<N; jj++) {
            tmp = A[r][jj];
            A[r][jj] = A[k][jj];
            A[k][jj] = tmp;
          }
          tmp  = B[k];
          B[k] = B[r];
          B[r] = tmp;

          // Substract lines
          for (int i=0; i<M; i++) if (i!=r) {
            double t = A[i][j];
            for (int jj=0; jj<N; jj++) {
              A[i][jj] -= t*A[r][jj];
            }
            B[i] -= t*B[r];
          }
          
        }

    }

    // Compute res, A is triangular now 
    for (int i=M-1;i>=0; i--) {
      res[i] = B[i]/A[i][i]; // can be -1 on last line
      for (int j=i+1;j<N;j++) {
        res[i] -= res[j]*A[i][j];
      }
    }


  }

  return res;

}

int main () 
{

  int N = 5;

  matrix mat(N);
  for (int i=0; i<N; i++) {
    mat[i] = rvec(N,1.);
  }
  mat[0][4] = 5.;
  mat[1][0] = 6.;
  mat[2][4] = 7.;
  mat[3][2] = 8.;
  mat[4][1] = 9.;

  rvec rhs(N); 
  rhs[0] = -4.;
  rhs[1] = 9.;
  rhs[2] = -8.;
  rhs[3] = 4.;
  rhs[4] = 28.;

  // sol is supposed to be x = (1 3 0 2 -2);
  rvec x = solveGauss(mat,rhs);

  for (int i=0; i<N; i++)
    std::cout << x[i] << " ";
  std::cout << std::endl;

}
 



